package converter;

import java.util.ArrayList;
import java.util.Iterator;

import VOCLModel.AssociationEndRole;
import VOCLModel.ClassifierNode;

/**
 * An instance of this class represents a path in a class diagram which is part of a VOCL diagram. It holds the actual
 * and the start node and a list of AssociationEndRoles (from VOCLModel3) pointing on the visited classifier roles.
 * 
 * @author melodica
 */
public class GraphPath {
	private ClassifierNode source;
	private ClassifierNode actual;
	private ArrayList links;

	/**
	 * @param graph
	 *            A GraphPath to be copied
	 */
	public GraphPath(GraphPath graph) {
		source = graph.getSource();
		actual = graph.getActual();
		links = new ArrayList(graph.links);
	}

	/**
	 * @param start
	 *            The classifier role that is the start node of the new GraphPath
	 */
	public GraphPath(ClassifierNode start) {
		source = start;
		actual = start;
		links = new ArrayList();
	}

	/**
	 * @param assER
	 *            An AssociationEndRole that will be inserted in the list of visited AssociationEndRoles of this path.
	 */
	public void addLink(AssociationEndRole assER) {
		links.add(assER);
	}

	/**
	 * This methos returns whether this path is actually ending in its start node.
	 */
	public boolean isSelfCycle() {
		return (source == actual);
	}

	/**
	 * This method returns whether this path describes a cycle. If two AssociationEndRoles in its list point to the same
	 * classifier role then the path is a cycle.
	 */
	public boolean isCycle() {
		Iterator ilinks = links.iterator();
		AssociationEndRole actLink;

		while (ilinks.hasNext()) {
			actLink = (AssociationEndRole) ilinks.next();

			// Wenn eine EndRole auf die aktuellen ClassifierRole
			// und nicht die letzte ist, dann ist ein Kreislauf vorhanden
			if (ilinks.hasNext() && (actLink.getType() == actual)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * @param path
	 *            A path to be checked if it is covered by this path.
	 */
	public boolean isSuperPathOf(GraphPath path) {
		int ownsize = links.size();
		int othersize = path.links.size();

		if ((ownsize < othersize) || (source != path.source)) {
			return false;
		}

		for (int i = 0; i <= (othersize - 1); i++) {
			if (links.get(i) != path.getLink(i)) {
				return false;
			}
		}

		return true;

	}

	/**
	 * @param paths
	 *            A list of paths to be checked for containing the actual node of this path.
	 * @return An alternating list of GraphPaths and AssociationEndRoles. A path in this list contains the actual node
	 *         of this path followed by the AssociationEndRole pointing to this node for convenience.
	 */
	public ArrayList pathsIncludingActual(ArrayList paths) {
		ArrayList pathLinkList = new ArrayList();
		GraphPath actPath;

		// Druchsuche alle Pfade in der Liste
		for (Iterator ipaths = paths.iterator(); ipaths.hasNext();) {
			actPath = (GraphPath) ipaths.next();

			// ob ein Link auf die aktuellen ClassifierRole zeigt
			AssociationEndRole actLink;
			for (Iterator ilinks = actPath.links.iterator(); ilinks.hasNext();) {
				actLink = (AssociationEndRole) ilinks.next();

				// Wenn eine EndRole auf die aktuellen ClassifierRole
				// Pfad UND letzten Link in Liste speichern (nicht mehr!)
				if (actLink.getType() == actual) {
					pathLinkList.add(actPath);
					//pathLinkList.add(actLink);
				}
			}
		}

		return pathLinkList;
	}

	public ClassifierNode getActual() {
		return actual;
	}

	public ClassifierNode getSource() {
		return source;
	}

	/**
	 * @return The length of the AssociationEndRole list.
	 */
	public int length() {
		return links.size();
	}

	/**
	 * @param i
	 *            The element of the AssociationEndRole list to be returned
	 */
	public AssociationEndRole getLink(int i) {
		if (i <= (links.size() - 1)) {
			return (AssociationEndRole) links.get(i);
		}

		return null;
	}

	public void setActual(ClassifierNode role) {
		actual = role;
	}

	/**
	 * This method is for testing only. It prints a textual description of this path.
	 */
	public void show() {
		System.out.print("Path: self");

		for (Iterator i = links.iterator(); i.hasNext();) {
			System.out.print("->" + ((AssociationEndRole) i.next()).getName());
		}
	}
}