/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.action.request.VOCLContainerPropertiesRequest;
import vocleditor.commands.adds.AddConditionCommand;
import vocleditor.figure.ConditionFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.Condition;
import VOCLModel.VariableDeclaration;



/**
 * Class       :ConditionEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for Condition
 * <br/>
 * @author vilapower
 */
public class ConditionEditPart extends AbstractGraphicalEditPart implements Adapter  {
	
	private IPropertySource propertySource = null;
	private Notifier target;
	
	/**
	 * Creates a new ConditionEditPart instance.
	 * @param element Condition
	 */
	public ConditionEditPart(Condition condition) {
		setModel(condition);
	}


	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure(){	
		Condition condition= getCondition();
		VariableDeclaration left=condition.getLeft();
		VariableDeclaration right=condition.getRight();
		String strCondition="( " +left.getVarName()
							 + condition.getOp() ;
		if(condition.getValue().trim().length()!=0)
			strCondition=strCondition + condition.getValue();
		else
			strCondition=strCondition + right.getVarName();
		strCondition=strCondition + " )" ;
			
 		return new ConditionFigure(strCondition);
	}
	
	/**
	 * returns Figure of this EditPart
	 * @return ConditionFigure
	 */
	protected ConditionFigure getConditionFigure(){
		return (ConditionFigure) getFigure();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){}

	/** 
	 * Return the Model of this EditPart as Condition.
	 * @see VOCLModel3.Condition
	 *@return  Condition .
	 */
	public Condition getCondition(){return (Condition) getModel();}
    

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive())return;
		// start listening for changes in the model
		getCondition().eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive())return;
		// stop listening for changes in the model
		getCondition().eAdapters().remove(this);
		super.deactivate();
	}



	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {return target;}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {target = newTarget;}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	/**
	 * returns IPropertySource for the EditPart
	 * @return IPropertySource 
	 */		
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getCondition() );
		}
		return propertySource;	
	}
	
	/**
	*checks ,if the parameters of Condition were changed
	* @param notification org.eclipse.emf.common.notify.Notification
	*/	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		switch( type ) {
			case Notification.ADD:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
				refreshVisuals();
				break;
			case Notification.SET:
			case Notification.UNSET:
				refreshVisuals();
				break;
		}
	}
		
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */	
	protected void refreshVisuals() {
		Condition condition= getCondition();
		VariableDeclaration left=condition.getLeft();
		VariableDeclaration right=condition.getRight();
		String strCondition="( " +left.getVarName()
							 + condition.getOp() ;
		if(condition.getValue().trim().length()!=0)
			strCondition=strCondition + condition.getValue();
		else
			strCondition=strCondition + right.getVarName();
		strCondition=strCondition + " )";
		getConditionFigure().setLabel(strCondition);
	}

	
	/**
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	public boolean understandsRequest(Request req) {
		if (req instanceof VOCLContainerPropertiesRequest) return true;
		return super.understandsRequest(req);
	}

	/**
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	public Command getCommand(Request request) {
		if (request instanceof VOCLContainerPropertiesRequest) return new AddConditionCommand();
		return super.getCommand(request);
	}
	
	/**
	 * Condition can be never selected
	 * @see org.eclipse.gef.EditPart#setSelected(int)
	 */
	public void setSelected(int value) {}


}
