/**
 * (c) Copyright group Vilapower
 *
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.tfs.cs.tu-berlin.de/vila
 * 
 * Contributors:
 *     	Eduard Weiss
 * 		Tony Modica
 * 		Frank Hermann
 * 		Holger Endert
 * 
 */
package vocleditor.edit;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policy.VOCLDiagrammXYLayoutEditPolicy;
import vocleditor.edit.policy.VOCLEditPolicy;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.VOCLDiagramm;



/**
 * Class       :VOCLContainerEditPart<br/>
 * Package     : vocleditor.edit<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Class to provide application for VOCLDiagramm
 * <br/>
 * @author vilapower
 */
public class VOCLDiagrammEditPart extends AbstractGraphicalEditPart implements Adapter  {
	
	private IPropertySource propertySource = null;
	private Notifier target;
	
	/**
	 * Creates a new VOCLContainerEditPart instance.
	 * @param element VOCLDiagramm
	 */
	public VOCLDiagrammEditPart(VOCLDiagramm vocldiagramm) {
		setModel(vocldiagramm);
	}


	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure(){
		FreeformLayer layer = new FreeformLayer();
		layer.setLayoutManager(new FreeformLayout());
		layer.setBorder(new LineBorder(1));
		return layer;

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies(){
		installEditPolicy(EditPolicy.CONTAINER_ROLE, new VOCLEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new VOCLDiagrammXYLayoutEditPolicy());	
	}

	/** 
	 * Return the Model of this EditPart as VOCLDiagramm.
	 * @see VOCLModel3.VOCLDiagramm
	 *@return  VOCLDiagramm .
	 */
	public VOCLDiagramm getVOCLDiagramm(){
		return (VOCLDiagramm)getModel();
	}
    
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren(){
		List	allChildren=super.getModelChildren();
		if(allChildren==null || allChildren.size()==0) 
			allChildren= new LinkedList();
		
		Iterator it;  	
		it = getVOCLDiagramm().getConstraint().iterator();
		while(it.hasNext()) allChildren.add( it.next() );	
		return allChildren;
	}

	
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate(){
		if (isActive())return;
		// start listening for changes in the model
		getVOCLDiagramm().eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate(){
		if (!isActive())return;
		// stop listening for changes in the model
		getVOCLDiagramm().eAdapters().remove(this);
		super.deactivate();
	}



	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		/* override the default behavior defined in AbstractEditPart
		*  which would expect the model to be a property sourced. 
		*  instead the editpart can provide a property source
		*/
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/**
	 * returns IPropertySource for the EditPart
	 * @return IPropertySource 
	 */		
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getVOCLDiagramm() );
		}
		return propertySource;
		
	}
	
	/**
	* checks if the model was be changed
	* @param notification org.eclipse.emf.common.notify.Notification
	* 
	*/	
	public void notifyChanged(Notification notification) {
		int type = notification.getEventType();
		switch( type ) {
			case Notification.ADD:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
				refreshChildren();
				refreshVisuals();
				break;
			case Notification.SET:
				refreshVisuals();
				break;
		}
		
		
	}

}
